import { AbstractAstNode } from '@fern-api/generator-commons';
import { ModulePath, AttrPath } from './ast/core/types.cjs';
import { Writer } from './ast/core/Writer.cjs';

declare abstract class AstNode extends AbstractAstNode {
    protected references: Reference[];
    addReference(reference: Reference): void;
    inheritReferences(astNode: AstNode | undefined): void;
    getReferences(): Reference[];
    /**
     * Writes the node to a string.
     */
    toString(): string;
    /**
     * Writes the node to a string and formats it.
     */
    toStringFormatted(): Promise<string>;
}

declare class Type extends AstNode {
    private internalType;
    private constructor();
    static int(): Type;
    static float(): Type;
    static bool(): Type;
    static str(): Type;
    static bytes(): Type;
    static list(value: Type): Type;
    static set(value: Type): Type;
    static tuple(values: Type[]): Type;
    static dict(keyType: Type, valueType: Type): Type;
    static none(): Type;
    static uuid(): Type;
    static optional(value: Type): Type;
    static union(values: Type[]): Type;
    static any(): Type;
    static reference(value: Reference): Type;
    write(writer: Writer): void;
}

declare namespace Reference {
    interface Args {
        name: string;
        modulePath?: ModulePath;
        genericTypes?: Type[];
        alias?: string;
        attribute?: AttrPath;
    }
}
declare class Reference extends AstNode {
    readonly name: string;
    readonly modulePath: ModulePath;
    private readonly genericTypes;
    readonly alias: string | undefined;
    private readonly attribute;
    constructor({ name, modulePath, genericTypes, alias, attribute }: Reference.Args);
    write(writer: Writer): void;
    getFullyQualifiedModulePath(): string;
}

export { AstNode as A, Reference as R, Type as T };
